==> gdeskcal-0.30.tgz: share/gnome/gdeskcal/code/CalWindow.py <==
from Observable import Observable
import configvalues as conf
import desktop
import errormessage

import gtk



#
# Class for the calendar window.
#
class CalWindow(gtk.Window, Observable):

    # messages to the observers
    OBS_WHEEL = 0
    OBS_CLOSE = 1
    OBS_MOVE = 2
    OBS_MENU = 3
    

    #
    # Constructor.
    #
    def __init__(self):

        # flag indicating whether window is currently being dragged
        self.__dragging = 0
        self.__drag_offset = (0, 0)

        # flag indicating whether transparency is enabled
        self.__transparency = 1

        self.__bg_alpha = 255
        self.__bg_image = None
        self.__bg_root = None
        self.__bg_image_file = ""

        self.__position = (0, 0)


        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.set_title("gDeskCal:" + str(hash(self)))
        self.set_size_request(10, 10)
        self.set_decorated(gtk.FALSE)
        self.stick()
        
        self.connect("delete-event", self.__on_close)
        self.connect("scroll-event", self.__on_mouse_wheel)
        self.connect("key-press-event", self.__on_key)
        self.connect("expose-event", self.__on_expose)
        self.connect("configure-event", self.__on_configure)

        # window moving
        self.connect("button-press-event", self.__on_button_press)
        self.connect("button-release-event", self.__on_button_release)

        self.add_events(gtk.gdk.SCROLL_MASK | gtk.gdk.POINTER_MOTION_MASK |
                        gtk.gdk.POINTER_MOTION_HINT_MASK |
                        gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK)

        # layout
        self.__layout = gtk.Fixed()
        self.__layout.show()
        self.add(self.__layout)

        self.show()
        desktop.skip_taskbar(self)



    #
    # Sets the background image of the window.
    #
    def set_bg_image(self, filename, alpha):

        if (filename and filename == self.__bg_image_file): return
        self.__bg_image_file = filename

        if (filename):
            # this is an ugly hack for avoiding memory leaks in pygtk
            img = gtk.Image()
            img.set_from_file(filename)
            self.__bg_image = img.get_pixbuf()
            
            # this would cause memory leaks in pygtk versions < 1.99.14
            #self.__bg_image = gtk.gdk.pixbuf_new_from_file(filename)
                

            self.__bg_alpha = alpha
                
        if (not filename or not self.__bg_image):
            self.__bg_image = gtk.gdk.Pixbuf(0, 1, 8, 10, 10)
            self.__bg_image.fill(0xffffffff)
            self.__bg_alpha = 0

        self.__display_bg()




    #
    # Displays the background image.
    #
    def __display_bg(self):

        # transparency effect
        width = self.__bg_root.get_width()
        height = self.__bg_root.get_height()

        # we cannot use Pixbuf.copy() here because of a memory leak
        # bug in pygtk versions < 1.99.14
        image = gtk.gdk.Pixbuf(0, 1, 8, width, height)
        self.__bg_root.copy_area(0, 0, width, height, image, 0, 0)
            
        self.__bg_image.composite(image, 0, 0, width, height,
                                  0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR,
                                  self.__bg_alpha)
            

        # set bg
        pix, mask = image.render_pixmap_and_mask()

        style = gtk.Style()
        style.bg_pixmap[gtk.STATE_NORMAL] = pix
        self.set_style(style)
        self.queue_draw()




    #
    # Updates the background for transparency.
    #
    def update_bg(self, nohack = 0):

        x, y = self.window.get_origin()
        width, height = self.window.get_size()
        if (self.__transparency and width > 10 and height > 10):
            self.__capture_bg(x, y, width, height, nohack)
            self.__display_bg()



    #
    # Captures the background to create transparency.
    #
    def __capture_bg(self, x, y, width, height, nohack = 0):

        # try capturing the background
        try:
            self.__bg_root = desktop.get_wallpaper(x, y, width, height)

        # use the old transparency hack if capturing has failed
        except:
            if (nohack): return  # must not use the dirty hack

            errormessage.error(_("Could not access background image. " \
                                 "Screen capturing will be used instead."))

            self.hide()
            self.move(-1000, 0)  # Metacity doesn't like this
            self.queue_draw()

            # wait long enough for the window to disappear; does this work
            # always?
            while (gtk.gdk.events_pending()): gtk.mainiteration()
            gtk.mainiteration()
        
            self.__bg_root = desktop.get_wallpaper_fallback(x, y,
                                                            width, height)
            self.show()
            self.move(x, y)

        self.stick()
        desktop.skip_taskbar(self)




    #
    # Reacts on mouse wheel.
    #
    def __on_mouse_wheel(self, src, event):

        if (event.direction == gtk.gdk.SCROLL_UP): step = -1
        elif (event.direction == gtk.gdk.SCROLL_DOWN): step = 1
        else: step = 0
        self.update_observer(self.OBS_WHEEL, step)



    #
    # Reacts on key presses.
    #
    def __on_key(self, src, event):

        key = event.keyval
        if (key == 32):
            self.update_bg()



    #
    # Keeps the window below others.
    #
    def __on_expose(self, src, event):

        # funny thing happens when a DeskCal is covering another
        self.window.lower()



    #
    # Reacts on moving the window.
    #
    def __on_configure(self, src, event):

        pos = self.window.get_origin()
        if (pos != self.__position):
            self.update_observer(self.OBS_MOVE, pos)
            self.__position = pos

            self.update_bg(nohack = 1)
            
        


    #
    # Reacts on closing the window.
    #
    def __on_close(self, src, event):

        self.update_observer(self.OBS_CLOSE)



    #
    # Reacts on pressing a mouse button.
    #
    def __on_button_press(self, src, event):

        button = event.button
        modifiers = event.state
        x, y = self.get_pointer()
        if (button == 3):
            self.update_observer(self.OBS_MENU, button, event.time)

        #if ((modifiers & gtk.gdk.MOD1_MASK) and
        #  not self.__dragging and button == 1):
        #    self.move_window()



    #
    # Reacts on releasing a mouse button.
    #
    def __on_button_release(self, src, event):

        if (self.__dragging): self.__dragging = 0



    #
    # Reacts on moving the mouse pointer.
    #
    def __on_move(self):

        if (self.__dragging):
            offx, offy = self.__drag_offset
            winx, winy = self.get_position()
            x, y = self.get_pointer()
            x += winx; y += winy
            self.move(x - offx, y - offy)

            return gtk.TRUE

        else:
            return gtk.FALSE



    #
    # Puts or moves a child widget at the given position.
    #
    def put(self, widget, x, y):

        if (not widget in self.__layout.get_children()):
            self.__layout.put(widget, x, y)
        else:
            self.__layout.move(widget, x, y)



    #
    # Switches to window move mode.
    #
    def move_window(self):

        x, y = self.get_pointer()
        width, height = self.window.get_size()
        x = max(4, min(x, width - 4))
        y = max(4, min(y, height - 4))
        self.__dragging = 1
        self.__drag_offset = (x, y)
        gtk.timeout_add(20, self.__on_move)

        



    #
    # Configures the window.
    #
    def set_config(self, config):

        x = config.get([conf.GEOMETRY, conf.WINDOW, conf.X])
        y = config.get([conf.GEOMETRY, conf.WINDOW, conf.Y])

        width = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW, conf.WIDTH])
        height = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW,
                             conf.HEIGHT])

        self.__transparency = config.get([conf.TEMPORARY, conf.TRANSPARENCY])

        self.move(x, y)
        self.resize(width, height)
        self.queue_draw()
        
        if (self.__transparency):
            self.__capture_bg(x, y, width, height)
        else:
            self.__bg_root = gtk.gdk.Pixbuf(0, 1, 8, width, height)
            # TODO: let the user set the bg color
            self.__bg_root.fill(0xddddeeff)

==> gdeskcal-0.53.tgz: share/gnome/gdeskcal/code/CalWindow.py <==
from GlassWindow import GlassWindow
from Observable import Observable
import configvalues as conf
import desktop
import errormessage

import gtk



#
# Class for the calendar window.
#
class CalWindow(GlassWindow, Observable):

    # messages to the observers
    OBS_WHEEL = 0
    OBS_CLOSE = 1
    OBS_MOVE = 2
    OBS_MENU = 3
    

    #
    # Constructor.
    #
    def __init__(self):

        # flag indicating whether this window is currently being dragged
        self.__dragging = 0
        self.__drag_offset = (0, 0)

        # remember the current window position to detect movements
        self.__position = (0, 0)


    	GlassWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
        import icon; self.set_icon(icon.ICON)
        self.set_title("gDeskCal")
        self.set_decorated(gtk.FALSE)
        self.stick()
        
        self.connect("delete-event", self.__on_close)
        self.connect("scroll-event", self.__on_mouse_wheel)
        self.connect("key-press-event", self.__on_key)
        #self.connect("expose-event", self.__on_expose)
        gtk.timeout_add(250, self.__on_expose, None, None)
        self.connect("configure-event", self.__on_configure)

        # window moving
        self.connect("button-press-event", self.__on_button_press)
        self.connect("button-release-event", self.__on_button_release)

        self.add_events(gtk.gdk.SCROLL_MASK | gtk.gdk.POINTER_MOTION_MASK |
                        gtk.gdk.POINTER_MOTION_HINT_MASK |
                        gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK)

        # layout
        self.__layout = gtk.Fixed()
        self.__layout.show()
        self.add(self.__layout)

        # yeah, finally!
        gtk.idle_add(self.set_property, "skip-taskbar-hint", 1)
        gtk.idle_add(self.set_property, "skip-pager-hint", 1)



    #
    # Reacts on mouse wheel.
    #
    def __on_mouse_wheel(self, src, event):

        if (event.direction == gtk.gdk.SCROLL_UP): step = -1
        elif (event.direction == gtk.gdk.SCROLL_DOWN): step = 1
        else: step = 0
        self.update_observer(self.OBS_WHEEL, step)



    #
    # Reacts on key presses.
    #
    def __on_key(self, src, event):

        key = gtk.gdk.keyval_name(event.keyval)

        # SPACE updates the background
        if (key == "space"):
            self.update_bg()

        # F10 opens the menu
        elif (key == "F10"):
            self.update_observer(self.OBS_MENU, 0, event.time)



    #
    # Keeps the window below others.
    #
    def __on_expose(self, src, event):

        # funny thing happens when a DeskCal is covering another
        if (not self.__dragging): self.window.lower()
        
        return gtk.TRUE
    

    #
    # Reacts on moving the window.
    #
    def __on_configure(self, src, event):

        pos = self.window.get_origin()
        if (pos != self.__position):
            self.update_observer(self.OBS_MOVE, pos)
            self.__position = pos




    #
    # Reacts on closing the window.
    #
    def __on_close(self, src, event):

        self.update_observer(self.OBS_CLOSE)



    #
    # Reacts on pressing a mouse button.
    #
    def __on_button_press(self, src, event):

        button = event.button
        modifiers = event.state
        x, y = self.get_pointer()
        if (button == 3):
            self.update_observer(self.OBS_MENU, button, event.time)



    #
    # Reacts on releasing a mouse button.
    #
    def __on_button_release(self, src, event):

        if (self.__dragging):
            self.__dragging = 0
            self.update_bg()



    #
    # Reacts on moving the mouse pointer.
    #
    def __on_move(self):

        if (self.__dragging):
            offx, offy = self.__drag_offset
            winx, winy = self.get_position()
            x, y = self.get_pointer()
            x += winx; y += winy
            self.move(x - offx, y - offy)

            return gtk.TRUE

        else:
            return gtk.FALSE



    #
    # Updates the background.
    #
    def update_bg(self, noscreenshot = 0):

        GlassWindow.update_bg(self, noscreenshot)
        self.stick()
        


    #
    # Puts or moves a child widget at the given position.
    #
    def put(self, widget, x, y):

        if (not widget in self.__layout.get_children()):
            self.__layout.put(widget, x, y)
        else:
            self.__layout.move(widget, x, y)



    #
    # Switches to window move mode.
    #
    def move_window(self):

        x, y = self.get_pointer()
        width, height = self.window.get_size()
        x = max(4, min(x, width - 4))
        y = max(4, min(y, height - 4))
        self.__dragging = 1
        self.__drag_offset = (x, y)
        gtk.timeout_add(20, self.__on_move)

        



    #
    # Configures the window.
    #
    def set_config(self, config):

        x = config.get([conf.GEOMETRY, conf.WINDOW, conf.X])
        y = config.get([conf.GEOMETRY, conf.WINDOW, conf.Y])

        width = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW, conf.WIDTH])
        height = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW,
                             conf.HEIGHT])

        transparency = config.get([conf.TEMPORARY, conf.TRANSPARENCY])

        self.set_transparency_mode(transparency)
        self.show()
        self.hide()
        #gtk.idle_add(self.set_property, "skip-taskbar-hint", 1)
        #gtk.idle_add(self.set_property, "skip-pager-hint", 1)
        self.show()

        self.move(x, y)
        self.resize(width, height)
        self.set_size_request(width, height)
        
        if (transparency != self.TRANSPARENCY_NONE):
            self.update_bg()
